unit mfk3sys;

{$mode objfpc}{$H+}{$M+}

interface

uses
  Classes, SysUtils, dateutils;

type

    (* Verschiedene Enums, die ich benötige *)
    EBuchungsart = (baEinnahme, baAusgabe);
    EZahlungsart = (zaAbbuchung, zaUeberweisung, zaDauerauftrag, zaBar,
                    zaAbhebung);
    EZahlungsperiode = (zpMonat, zpZweiMonate, zpQuartal, zpHalbjahr, zpJahr);

    (* Exceptions, die ich benötige *)

    { MFKDatenfeldException }

    EBuchungException = class(Exception)
    end;




  (* Basisklasse für Buchungen *)
  TBuchung = class
     protected
        FName : String;
        FBuchungsart : EBuchungsart;
        FKategorie : String;
        FZahlungsart : EZahlungsart;
        FBemerkungen : String;
        FBetrag : Currency;
      public
        function BetragZumDatum(Datum : TDateTime) : Currency; virtual; abstract;
        property Name : String read FName;
        property Buchungsart : EBuchungsart read FBuchungsart;
        property Zahlungsart : EZahlungsart read FZahlungsart;
        property Kategorie : String read FKategorie;
        property Betrag : Currency read FBetrag;
        property Bemerkungen : String read FBemerkungen;

  end;

   { TBuchungEinmalig }

   TBuchungEinmalig = class(TBuchung)
      private
        FDatum : TDateTime;
        procedure setBetrag(AValue: Currency);
        procedure SetDatum(AValue: TDateTime);
        procedure setName(AName : String);
        procedure setKategorie(AKategorie : String);
      public
        property Name : String read FName write setName;
        property Buchungsart : EBuchungsart read FBuchungsart write FBuchungsart;
        property Zahlungsart : EZahlungsart read FZahlungsart write FZahlungsart;
        property Kategorie : String read FKategorie write setKategorie;
        property Betrag : Currency read FBetrag write setBetrag;
        property Datum : TDateTime read FDatum write SetDatum;
        property Bemerkungen : String read FBemerkungen write FBemerkungen;
        function BetragZumDatum(ADatum: TDateTime): Currency; override;


   end;

  (* Utilityfunktionen *)


implementation

function vergleicheMFKDatum(Datum1: TDateTime; Datum2: TDateTime): Boolean;
begin
  Result := (MonthOf(Datum1) = MonthOf(Datum2))
            and (YearOf(Datum1) = YearOf(Datum2));
end;


{ TBuchungEinmalig }

procedure TBuchungEinmalig.setBetrag(AValue: Currency);
var Meldung : String;
begin
  if AValue = FBetrag then Exit;
  Meldung := 'Betrag darf nicht negativ sein!';
  if AValue < 0 then
    raise EBuchungException.Create(Meldung)
  else
    FBetrag:= AValue;
end;

procedure TBuchungEinmalig.SetDatum(AValue: TDateTime);
begin
  if FDatum=AValue then Exit;
  FDatum:=AValue;
end;

procedure TBuchungEinmalig.setName(AName: String);
var Meldung : String;
begin
  Meldung:= 'Name einer Buchung darf nicht leer sein';
  if AName = '' then
    raise EBuchungException.Create(Meldung)
  else
    FName:= AName;
end;


procedure TBuchungEinmalig.setKategorie(AKategorie: String);
var meldung : String;
begin
  Meldung := 'Kategorie einer Buchung darf nicht leer sein';
  if AKategorie = '' then
    raise EBuchungException.Create(Meldung)
  else
    FKategorie:= AKategorie;
end;


function TBuchungEinmalig.BetragZumDatum(ADatum: TDateTime): Currency;
begin
  if IsSameMonth(FDatum, ADatum) then
    Result := FBetrag
  else
    Result := 0;
end;



end.

